/*
 * FILE --- rls_estm_ex.cc 
 *
 * PURPOSE --- Main source file that provides an example usage of the 
 * usage of the Linear State Observer utility for Single-Input-Single-Output 
 * (SISO) plants. It also provides a simulation of an 
 * example plant whose state is to be observed. In actual implementations 
 * the signal generated from this example will be measured from actual systems.
 *
 */

// Include Files
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "rls_estm.hh"

void main(void) 
{
  // Implements example H(z) = K/(z^3 - az^2 - bz - c)
  // with unknown parameters --- K, a, b, c

  // System variables
  double u = 0.0, y = 0.0, y_1 = 0.0, y_2 = 0.0, y_3 = 0.0; 

  // System parameters
  const double K = 256.3, a = 0.55, b = 0.43, c = - 0.0474;

  // Regression vector
  double phi[4] = {0.0, 0.0, 0.0, 0.0};

  // Estimate of the parameters
  double theta[4] = {0.0, 0.0, 0.0, 0.0};
  
  // Estimate of the system output
  double  y_hat = 0.0;

  // The RLS estimator 
  RLS_ESTM my_rls(4); // Four parameters to be estimated --- K, a, b, c

  // Maximum # of iterations
  int max_itearions = 20;

  FILE *fp;  // To save data

  // Set the covariance matrix to 2000*Identity
  my_rls.set_P(2000.0);
  
  // Set the initial parameter estimate
  my_rls.set_theta(theta);

  // Set no forgeting factor
  my_rls.set_lambda(1.0);

  // Open the file
  if (!(fp = fopen("rls_estm_out.dat","w"))) {
    printf("Can't open file.\n");
    exit(1);
  }

  // Simulate for max_itearions
  for(int ii=0; ii < max_itearions; ii++)
    {
      // Next sampling period
      y_3 = y_2;
      y_2 = y_1; 
      y_1 = y; 

      // Calculate the new control
      //      u = sin(0.01*ii) + sin(0.1*ii);
      u = 1.0;

      // Iterate the plant
      y = K*u + a*y_1 + b*y_2 + c*y_3;
      
      // Save the data
      fprintf(fp,"%f %f %f %f %f, %f, %f\n",
	      u, y, y_hat, theta[0], theta[1], theta[2], theta[3]);

      // Form the regression vector
      phi[0] = u; 
      phi[1] = y_1; 
      phi[2] = y_2; 
      phi[3] = y_3;
      
      // Calculate the new estimate
      y_hat = my_rls.estimate(y, phi, theta);
    } 
  
  fclose(fp);
}

